import React from 'react'
import { waitFor, renderHook } from '@testing-library/react'
import { AuthProvider, useAuth } from '../context'
import { TEST_CONFIG } from './mocks/testConfig'

let location: Location

describe('AuthProvider', () => {
  beforeEach(() => {
    location = window.location
    jest.spyOn(window, 'location', 'get').mockRestore()
  })

  afterEach(() => {
    jest.clearAllMocks()
  })

  test('Login function will navigate to the correct url', async () => {
    const mockedLocation = {
      ...location,
      assign: jest.fn(),
    }
    jest.spyOn(window, 'location', 'get').mockReturnValue(mockedLocation)

    const wrapper = ({ children }: any) => <AuthProvider {...TEST_CONFIG}>{children}</AuthProvider>
    const { result } = renderHook(() => useAuth(), {
      wrapper,
    })

    await waitFor(() => result.current.login())

    const expectedUrl =
      'http://localhost:9000/web/login/init?client_id=00000000&redirect_uri=http%3A%2F%2Flocalhost%3A8000'

    await waitFor(() => expect(mockedLocation.assign).toBeCalledWith(expectedUrl))
  })

  test('Logout function will navigate to the correct url', async () => {
    const mockedLocation = {
      ...location,
      assign: jest.fn(),
    }
    jest.spyOn(window, 'location', 'get').mockReturnValue(mockedLocation)

    const wrapper = ({ children }: any) => <AuthProvider {...TEST_CONFIG}>{children}</AuthProvider>
    const { result } = renderHook(() => useAuth(), {
      wrapper,
    })

    await result.current.logout()

    const expectedUrl =
      'http://localhost:9000/web/logout/init?client_id=00000000&redirect_uri=http%3A%2F%2Flocalhost%3A8000'

    await waitFor(() => expect(mockedLocation.assign).toBeCalledWith(expectedUrl))
  })

  test('Register function will navigate to the correct url', async () => {
    const mockedLocation = {
      ...location,
      assign: jest.fn(),
    }
    jest.spyOn(window, 'location', 'get').mockReturnValue(mockedLocation)

    const wrapper = ({ children }: any) => <AuthProvider {...TEST_CONFIG}>{children}</AuthProvider>
    const { result } = renderHook(() => useAuth(), {
      wrapper,
    })

    await waitFor(() => result.current.register())

    const expectedUrl =
      'http://localhost:9000/web/register/init?client_id=00000000&redirect_uri=http%3A%2F%2Flocalhost%3A8000'
    await waitFor(() => expect(mockedLocation.assign).toBeCalledWith(expectedUrl))
  })
})
