import { Dispatch, PropsWithChildren, SetStateAction } from 'react'

export interface IAuthContext {
  login: () => void
  logout: () => void
  register: () => void
  user?: unknown
  isUserLoading?: boolean
  isAuthenticated: boolean
  refreshToken?: () => void
}

export interface AuthConfig extends PropsWithChildren {
  clientID: string
  serverUrl: string
  postLogoutRedirectUrl?: string
  postLoginRedirectUrl?: string
  postRegisterRedirectUrl?: string
  onError?: Error
}

export enum ServerEndpointType {
  login = '/login/init',
  logout = '/logout/init',
  register = '/register/init',
  tokenRefreshWithRedirect = '/refresh/init',
  tokenRefresh = '/token/refresh',
  me = '/me',
}

export interface LoginModal {
  setIsOpen: Dispatch<SetStateAction<boolean>>
  login: () => void
  logout: () => void
}

export enum ERRORS {
  INIT = 'INIT',
  USER = 'USER',
  REFRESH_TOKEN = 'REFRESH_TOKEN',
}

export type Error = (error: ERRORS, value: unknown) => void
