import Cookies from 'js-cookie'
import { Dispatch, SetStateAction, useEffect, useRef } from 'react'
import { ACCESS_TOKEN_EXPIRY_COOKIE, REFRESH_TOKEN_EXPIRY_COOKIE } from '../constants'
import useBrowserTabActive from './useBrowserTabActive'
import { getRefreshAccessTokenInterval, refreshAccessTokenWithRedirect } from '../utils'

interface RefreshToken {
  setIsAuthenticated: Dispatch<SetStateAction<boolean>>
  refreshToken: () => void
  serverUrl: string
  clientID: string
}

const useRefreshToken = ({ setIsAuthenticated, refreshToken, serverUrl, clientID }: RefreshToken) => {
  const intervalRef = useRef<null | ReturnType<typeof setInterval>>(null)
  const isTabActive = useBrowserTabActive()

  useEffect(() => {
    const refreshTokenExpiryCookie = Cookies.get(REFRESH_TOKEN_EXPIRY_COOKIE)
    const accessTokenExpiryCookie = Cookies.get(ACCESS_TOKEN_EXPIRY_COOKIE)
    let timerRef = intervalRef.current

    if (isTabActive && refreshTokenExpiryCookie && !accessTokenExpiryCookie) {
      refreshAccessTokenWithRedirect({ serverUrl, clientID })
    }

    if (isTabActive && refreshTokenExpiryCookie && accessTokenExpiryCookie) {
      if (timerRef) {
        clearInterval(timerRef)
      }

      timerRef = setInterval(() => {
        void refreshToken()
      }, getRefreshAccessTokenInterval())
    }

    return () => {
      timerRef && clearInterval(timerRef)
    }
  }, [setIsAuthenticated, isTabActive, refreshToken, clientID, serverUrl])
}

export default useRefreshToken
