import Cookies from 'js-cookie'
import { useEffect, useState } from 'react'
import { ACCESS_TOKEN_EXPIRY_COOKIE } from '../constants'
import { getUser } from '../actions'
import { ERRORS, Error } from '../context/models'

interface User {
  serverUrl: string
  clientID: string
  onError: Error
}

const useUser = ({ serverUrl, clientID, onError }: User) => {
  const [isUserLoading, setIsUserLoading] = useState(false)
  const [user, setUser] = useState<unknown>({})

  useEffect(() => {
    if (!Cookies.get(ACCESS_TOKEN_EXPIRY_COOKIE)) {
      return
    }

    getUser({ serverUrl, clientID })
      .then((data) => {
        if (data) {
          setUser(user)
        }
      })
      .catch((error) => {
        onError(ERRORS.USER, error)
      })
      .finally(() => {
        setIsUserLoading(false)
      })
  }, [setIsUserLoading, setUser, user, serverUrl, clientID, onError])

  return { isUserLoading, user }
}

export default useUser
